#include "W4LauncherDialog.h"
#include "W4ModItem.h"
#include <Windows.h>

//#define DEBUGMODE 1

#include <QApplication>
#include <QMessageBox>
#include <QSettings>
#include <QDir>
#include <QDirIterator>
#include <QTextStream>
#include <QTextEdit>
#include <QDesktopServices>
#include <QProgressDialog>
#include <QProcess>

#ifdef DEBUGMODE
QTextEdit* dbg;
#endif

#define cs(b) (b?(Qt::Checked):(Qt::Unchecked))

void print(QString txt)
{
    #ifdef DEBUGMODE
    dbg->append(txt);
    #endif
}

// Quick functions for getting the width and height from a resolution "WIDTHxHEIGHT", and the frequency from a refresh rate "freqHz"
int getResX(const QString& res) {return res.section("x", 0, 0).toInt();}
int getResY(const QString& res) {return res.section("x", 1).toInt();}
int getFreq(QString frq) {frq.chop(2);return frq.toInt();}

///////////////////////////////////////////////////////////////////////////////////////
// Standard main() function

int main(int argc, char *argv[])
{
	QApplication app(argc, argv);

	#ifdef DEBUGMODE
	dbg = new QTextEdit();
    dbg->resize(640, 480);
    dbg->show();
    #endif
    
	W4LauncherDialog *dialog = new W4LauncherDialog(argc, argv);
	if(dialog->gamepath=="NONE") return 0;
    
	dialog->show();
	
	return app.exec();
}

///////////////////////////////////////////////////////////////////////////////////////
// Constructor
// Sets the interface up, loads the mods, etc...

W4LauncherDialog::W4LauncherDialog(int argc, char* argv[]) : QDialog()
{
    // Retrieve the game install path from the registry
    QSettings reg("HKEY_LOCAL_MACHINE\\SOFTWARE\\CODEMASTERS\\Worms 4 Mayhem", QSettings::NativeFormat);
    gamepath = reg.value("InstallDir", "NONE").toString();
    
    QString lang = reg.value("Language", "NONE").toString();
    if     (lang=="FRE") language_init(FRE, "W4LauncherDialog");
    else if(lang=="SPA") language_init(SPA, "W4LauncherDialog");
    else                 language_init(ENG, "W4LauncherDialog");
    
    if(gamepath == "NONE") // The key doesn't exist, game is probably not installed. Generate error message, and quit.
    {
        QMessageBox::critical(this, getlang(LANG_ERROR), getlang(LANG_ERROR_CANTFINDGAME));
        return;
    }
    
    // Setup the interface
	ui.setupUi(this);
	
	// Connect the signals and slots
	connect(ui.playButton     , SIGNAL(clicked()), this, SLOT(launchGame()));
	connect(ui.updateButton   , SIGNAL(clicked()), this, SLOT(checkUpdates()));
	connect(ui.upButton       , SIGNAL(clicked()), this, SLOT(moveUp()));
	connect(ui.downButton       , SIGNAL(clicked()), this, SLOT(moveDown()));
	connect(ui.cancelButton   , SIGNAL(clicked()), qApp, SLOT(quit()));
	connect(ui.resolutionCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(resolutionChanged()));
	connect(ui.modsList       , SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), this, SLOT(updateModInfo(QListWidgetItem*,QListWidgetItem*)));
	
	// Check if the WormsMods folder exists
	QDir dir(gamepath);
	if(!dir.exists("WormsMods"))
	{
	    // If it doesn't, just create it and don't go further
	    dir.mkdir("WormsMods");
	    return;
	}
	
	// Load launcher stuff
	
	QFile cfg(gamepath + "\\launcher.cfg");
	cfg.open(QIODevice::ReadOnly|QIODevice::Text);
	QTextStream ts0(&cfg);
	QString s;
	
	l_resX = 640;
	l_resY = 480;
	l_refresh = 60;
	l_nomovies = false;
	l_nosound = false;
	l_portfwd = true;
	l_allowss = false;
	l_mancam = false;
	l_telnet = false;
	
	do
	{
	    ts0 >> s;
	    
	    if     (s.startsWith("/W:"))        l_resX = s.mid(3).toUShort();
	    else if(s.startsWith("/H:"))        l_resY = s.mid(3).toUShort();
	    else if(s.startsWith("/REFRESH:"))  l_refresh = s.mid(9).toUShort();
	    else if(s=="/NOMOVIES")             l_nomovies = true;
	    else if(s=="/NOSOUNDS")             l_nosound = true;
	    else if(s=="/ENABLEPORTFORWARDING") l_portfwd = true;
	    else if(s=="/ALLOWSCREENSHOTS")     l_allowss = true;
	    else if(s=="/MANUALCAMERA")         l_mancam = true;
	    else if(s=="/TELNET")               l_telnet = true;
	} while(!s.isEmpty());
	cfg.close();
	ui.nomoviesCheck->setCheckState(cs(l_nomovies));
	ui.nosoundCheck->setCheckState(cs(l_nosound));
	ui.autoportsCheck->setCheckState(cs(l_portfwd));
	ui.allowscreensCheck->setCheckState(cs(l_allowss));
	ui.manualcamCheck->setCheckState(cs(l_mancam));
	ui.telnetCheck->setCheckState(cs(l_telnet));
	
	/*ui.resolutionCombo->addItem(QString::number(l_resX) + "x" + QString::number(l_resY));
	ui.refreshCombo->addItem(QString::number(l_refresh) + "Hz");*/
	
	// Get the list of available resolutions and refresh rates
	DEVMODE display;
	display.dmSize = sizeof(DEVMODE);
	for(int i=0;EnumDisplaySettings(NULL,i,&display);i++)
	{
	    int resX = display.dmPelsWidth;
        int resY = display.dmPelsHeight;
        int freq = display.dmDisplayFrequency;
        
        if(resX>=640 && resY>=480) // Seriously, would you ever play with a resolution lower than 640x480 ?
        {
            QString res = QString::number(resX)+"x"+QString::number(resY);
            QString rr = QString::number(freq);
            int index = resolutions.indexOf(res);
            if(index==-1) // Resolution not found, add it to the list
            {
                QStringList rrlist;
                rrlist << (rr+"Hz");
                
                resolutions.append(res);
                refreshRates.append(rrlist);
                ui.resolutionCombo->addItem(res);
            }
            else // Resolution has already been added, check the refresh rate
            {
                if(!refreshRates[index].contains(rr+"Hz")) // Refresh rate not found, add it to the list
                {
                    refreshRates[index].append(rr+"Hz");
                }
            }
        }
	}
	
	// Check the resolution and refresh rate given in launcher.cfg
	QString res = QString::number(l_resX)+"x"+QString::number(l_resY);
	QString rr = QString::number(l_refresh)+"Hz";
	int index = resolutions.indexOf(res);
	if(index==-1) // This resolution is not supported, select the nearest resolution
	{
	    ui.resolutionCombo->setCurrentIndex(0);
	    index = 0;
	    for(int i=resolutions.count()-1;i>=0;i--)
	    {
	        if(getResX(resolutions[i])<=l_resX && getResY(resolutions[i])<=l_resY)
	        {
	            ui.resolutionCombo->setCurrentIndex(i);
	            index = i;
	            break;
	        }
	    }
	    resolutionChanged();
	}
	else
    {
        ui.resolutionCombo->setCurrentIndex(index);
    }
    
    // Check the refresh rate
    QStringList rrlist = refreshRates[index];
    index = rrlist.indexOf(rr);
    if(index==-1) // This refresh rate is not supported, select the nearest refresh rate
    {
        ui.refreshCombo->setCurrentIndex(0);
        for(int i=rrlist.count()-1;i>=0;i--)
        {
            print(QString::number(getFreq(rrlist[i])));
            if(getFreq(rrlist[i])<=l_refresh)
            {
                ui.refreshCombo->setCurrentIndex(i);
                break;
            }
        }
    }
    else ui.refreshCombo->setCurrentIndex(index);
	
	// Load the mods
	dir.cd("WormsMods");
	
	QStringList moddirs = dir.entryList(QDir::Dirs); // Get the directory listing in WormsMods
	QListIterator<QString> it(moddirs);
	QString mdir;
	while (it.hasNext())
    {
        
        mdir = gamepath + "\\WormsMods\\" + it.next() + "\\";
        // Check if the info.txt file is there
        QFile info(mdir + "info.txt");
        
        if(!info.exists()) continue; // Else, it's not a mod, skip it.
        
        if(info.open(QIODevice::ReadOnly|QIODevice::Text)) // Read the mod info and populate the mods list.
        {
            W4ModItem* m = new W4ModItem(ui.modsList);
            m->dir = mdir;
            m->author = "N/A";
            m->version = "";
            m->desc = "N/A";
            m->priority = 0;
            
            QTextStream ts(&info);
            QString line;
            QStringList parts;
            do
            {
                line = ts.readLine();
                parts = line.split("=");
                if(parts.size()==2)
                {
                    if      (parts[0]=="name")     m->setText(parts[1]);
                    else if (parts[0]=="author")   m->author = parts[1];
                    else if (parts[0]=="version")  m->version = parts[1];
                    else if (parts[0]=="desc")     m->desc = parts[1];
                    else if (parts[0]=="priority") m->priority = parts[1].toUShort();
                }
            } while (!line.isNull());
            m->setText(m->text()+" "+m->version);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////////////
// Destructor. Do nothing for now...

W4LauncherDialog::~W4LauncherDialog()
{
    
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Launch the game with the chosen parameters

void W4LauncherDialog::launchGame()
{
    // Apply standard parameters
    
    l_resX = getResX(ui.resolutionCombo->currentText());
    l_resY = getResY(ui.resolutionCombo->currentText());
    l_refresh = getFreq(ui.refreshCombo->currentText());
    
    l_nomovies = (ui.nomoviesCheck->checkState()==Qt::Checked);
    l_nosound  = (ui.nosoundCheck->checkState() ==Qt::Checked);
    l_portfwd  = (ui.autoportsCheck->checkState()==Qt::Checked);
    l_allowss  = (ui.allowscreensCheck->checkState()==Qt::Checked);
    l_mancam   = (ui.manualcamCheck->checkState()==Qt::Checked);
    l_telnet   = (ui.telnetCheck->checkState()==Qt::Checked);
    
    QFile cfg(gamepath + "\\launcher.cfg");
    cfg.open(QIODevice::WriteOnly|QIODevice::Text);
	QTextStream out(&cfg);
	
	out << "/W:" << l_resX << " /H:" << l_resY << " /REFRESH:" << l_refresh;
	if(l_nomovies)  out << " /NOMOVIES";
	if(l_nosound)   out << " /NOSOUND";
	if(l_portfwd)   out << " /ENABLEPORTFORWARDING";
	if(l_allowss)   out << " /ALLOWSCREENSHOTS";
	if(l_mancam)    out << " /MANUALCAMERA";
	if(l_telnet)    out << " /TELNET";
	
	cfg.close();
    
    QStringList filter;
    filter << "*.twk";
    // First, count the total number of files to load
    
    unsigned int filecount=0;
    for(int i=0;i<ui.modsList->count();i++)
    {
        W4ModItem* mod = (W4ModItem*)ui.modsList->item(i);
        if(mod->checkState() == Qt::Unchecked) continue; // This mod is not selected, skip it
        QDir dir(mod->dir + "Tweak");
        if(dir.exists()) // Has a Tweak folder
        {
            QStringList twkdirs = dir.entryList(QDir::Dirs); // Get the list of folders in the Tweak directory (1 folder for each tweak file)
            QListIterator<QString> it(twkdirs);
            
	        while (it.hasNext())
            {
                QString twkname = it.next();
                if(!QFile::exists(gamepath + "\\data\\Tweak\\" + twkname + ".xml")) continue;
                dir.cd(twkname);
                QStringList twkfiles = dir.entryList(filter, QDir::Files);
                QListIterator<QString> it2(twkfiles);
                
                while (it2.hasNext())
                {
                    it2.next();
                    filecount++;
                }
                dir.cdUp();
            }
        }
    }
    
    QProgressDialog progress("LOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOLOL", 0, 0, filecount);
    if(filecount>0)
    {
        progress.setWindowModality(Qt::WindowModal);
        progress.show();
        progress.setLabelText("");
        qApp->processEvents();
    }
    filecount = 0;
    
    // Load the selected mods
    
    QStringList rtypes, rtypesdb;
    rtypes << "XIntResourceDetails"    << "XUintResourceDetails"   << "XFloatResourceDetails"
           << "XStringResourceDetails" << "XVectorResourceDetails" << "XColorResourceDetails";
    rtypesdb << "IntResources"    << "UintResources"   << "FloatResources"
             << "StringResources" << "VectorResources" << "ColorResources";
    for(int i=0;i<ui.modsList->count();i++)
    {
        W4ModItem* mod = (W4ModItem*)ui.modsList->item(i);
        
        if(mod->checkState() == Qt::Unchecked) continue; // This mod is not selected, skip it
        
        progress.setWindowTitle(mod->text());
        qApp->processEvents();
        //////////////////////////////////////////////////
        // Loading Tweaks                               //
        //////////////////////////////////////////////////
        
        QDir dir(mod->dir + "Tweak");
        if(dir.exists()) // Found a Tweak folder, process it
        {
            QStringList twkdirs = dir.entryList(QDir::Dirs); // Get the list of folders in the Tweak directory (1 folder for each tweak file)
            QListIterator<QString> it(twkdirs);
            
	        while (it.hasNext())
            {
                QString tweakname = it.next();
                QFile tweakfile(gamepath + "\\data\\Tweak\\" + tweakname + ".xml");
                if(!tweakfile.exists()) continue; // This tweak file does not exist. Next.
                
                progress.setLabelText(getlang(LANG_LOADINGTWEAK)+tweakname+".xml");
                qApp->processEvents();
                QFile tweakbackup(gamepath + "\\data\\Tweak\\" + tweakname + ".xml.backup");
                if(!tweakbackup.exists())
                {
                    // Create a backup file, if it has not been created yet by another mod.
                    tweakfile.copy(gamepath + "\\data\\Tweak\\" + tweakname + ".xml.backup");
                }
                
                
                
                
                QDomDocument docOld, doc;
                QDomElement xomObjectsOld, xomObjects;
                
                // Load data from original tweak file
                if (!xom2_load(docOld, xomObjectsOld, gamepath + "\\data\\Tweak\\" + tweakname + ".xml.backup"))
                {
                    QMessageBox::critical(this, getlang(LANG_ERROR), getlang(LANG_ERROR_CANTLOADDATA)+tweakname+".xml");
                    return;
                }
                // Load data from modified tweak
                if (!xom2_load(doc, xomObjects, gamepath + "\\data\\Tweak\\" + tweakname + ".xml"))
                {
                    QMessageBox::critical(this, getlang(LANG_ERROR), getlang(LANG_ERROR_CANTLOADDATA)+tweakname+".xml");
                    return;
                }
                // Get the tweak databank
                QDomElement databank = xom2_getObject(xomObjects, "id-0");
                
                
                // Now processing each .twk file
                dir.cd(tweakname);
                QStringList twkfiles = dir.entryList(filter, QDir::Files);
                QListIterator<QString> it2(twkfiles);
                
                bool noerrors = false;
                while (it2.hasNext())
                {
                    QString temp = it2.next();
                    QFile modfile(dir.path() + "/" +  temp);
                    QString objname = temp;
                    objname.chop(4);
                    if(!modfile.open(QIODevice::ReadOnly|QIODevice::Text)) continue;
                    QTextStream ts0(&modfile);
                    
                    QString data = ts0.readAll();
                    
                    // Simplify the program
                    data.remove(QRegExp("//[^\n]*\n|/\\*.*\\*/"));     // Remove commentaries
                    data.replace(QRegExp("\\n\\s*(\\S)"), "\n\\1");    // Remove useless spaces
                    data.replace(QRegExp("(\\S)[^\\S\\n]*="), "\\1="); // 
                    data.replace(QRegExp("=[^\\S\\n]*(\\S)"), "=\\1"); // 
                    data.replace(QRegExp("(\\S)\\s*\\n"), "\\1\n");    // 
                    data.replace(QRegExp("\\n\\s*\\n"), "\n");         // Remove empty lines
                    data.replace(QRegExp("[^\\S\\n]+"), " ");          // Replace multiple spaces by one space
                    data=data.trimmed();                               // Remove spaces at beginning and end of the program
                    modfile.close();
                    
                    if(objname=="resource")
                    {
                        QTextStream ts(&data,QIODevice::ReadOnly);
                        QString s;
                        unsigned char restype = 0;
                        do
                        {
                            s = ts.readLine();
                            if     (s.startsWith("#int"   ))  restype = 0;
                            else if(s.startsWith("#uint"  ))  restype = 1;
                            else if(s.startsWith("#float" ))  restype = 2;
                            else if(s.startsWith("#string"))  restype = 3;
                            else if(s.startsWith("#vector"))  restype = 4;
                            else if(s.startsWith("#color" ))  restype = 5;
                            else if(s.startsWith("#"      ))  {}
                            else
                            {
                                QString resource = s.section('=',0,0);
                                QString value    = s.section('=',1);
                                if(!resource.isEmpty())
                                {
                                    QDomElement o = xom2_getObject(xomObjects, resource);
                                    if(o.isNull())
                                    {
                                        o = xom2_createObject(xomObjects, resource, rtypes[restype]);
                                        xom2_forceAddProperty(databank, rtypesdb[restype], "@href "+resource);
                                    }
                                    xom2_setProperty(o, "Value", value);
                                    xom2_setProperty(o, "Name", resource);
                                    xom2_setProperty(o, "Flags", "64");
                                }
                            }
                        } while(!s.isNull());
                        progress.setValue(++filecount);
                        qApp->processEvents();
                        continue;
                    }
                    
                    QTextStream ts(&data,QIODevice::ReadOnly);
                    
                    
                    QDomElement object = xom2_getObject(xomObjects, objname);
                    
                    QString s;
                    if(object.isNull()) // Object does not exist, create it.
                    {
                        bool nodb = false;
                        short flags = -1;
                        bool created = false;
                        
                        while(1)
                        {
                            s=ts.readLine();
                            if(!created && s.startsWith("#class")) // #class keyword found : create a new object with a defined class
                            {
                                if(nodb) // #nodb flag found : do not add the object to the databank
                                {
                                    object = xom2_createObject(xomObjects, objname, s.mid(7));
                                }
                                else // add the object to the databank
                                {
                                    // Create the container resource for implementing the object in the databank
                                    QDomElement cont = xom2_createObject(xomObjects, objname, "XContainerResourceDetails");
                                    xom2_setProperty(cont, "Value", "@href "+objname+"-0");
                                    xom2_setProperty(cont, "Name" , objname);
                                    if(flags>=0 && flags<=255) xom2_setProperty(cont, "Flags", QString::number(flags));
                                    else                       xom2_setProperty(cont, "Flags", "80");
                                    // Create the object itself
                                    object = xom2_createObject(xomObjects, objname+"-0", s.mid(7));
                                    // Add the container in the databank
                                    xom2_forceAddProperty(databank, "ContainerResources", "@href "+objname);
                                }
                                created = true;
                            }
                            else if(!created && s.startsWith("#copy")) // #copy keyword found : copy an existing object
                            {
                                QDomElement ref = xom2_getObject(xomObjectsOld, s.mid(6));
                                if(!ref.isNull())
                                {
                                    if(nodb) // #nodb flag found : copy the object, but do not add it to the databank
                                    {
                                        object = xom2_cloneObject(xomObjects, ref, objname);
                                    }
                                    else // copy the object and add it to the databank
                                    {
                                        if(ref.tagName()=="XContainerResourceDetails") // The copied object is a container, copy its contents instead
                                        {
                                            QString href = xom2_getProperty(ref, "Value");
                                            if(href.startsWith("@href "))
                                            {
                                                href = href.mid(6);
                                                ref = xom2_getObject(xomObjectsOld, href);
                                            }
                                        }
                                        
                                        // Create the container resource for implementing the object in the databank
                                        QDomElement cont = xom2_createObject(xomObjects, objname, "XContainerResourceDetails");
                                        xom2_setProperty(cont, "Value", "@href "+objname+"-0");
                                        xom2_setProperty(cont, "Name" , objname);
                                        if(flags>=0 && flags<=255) xom2_setProperty(cont, "Flags", QString::number(flags));
                                        else                       xom2_setProperty(cont, "Flags", "80");
                                        // Create the object itself
                                        object = xom2_cloneObject(xomObjects, ref, objname+"-0");
                                        // Add the container in the databank
                                        xom2_forceAddProperty(databank, "ContainerResources", "@href "+objname);
                                    }
                                    created = true;
                                }
                            }
                            else if(s.startsWith("#nodb")) // #nodb keyword found : Do not add this object to the databank
                            {
                                nodb = true;
                            }
                            else if(s.startsWith("#flags")) // #flags keyword found : Overwrite the default container flags
                            {
                                bool ok;
                                flags = s.mid(7).toShort(&ok);
                                if(!ok) flags = -1;
                            }
                            else if(s.startsWith("#")) {}
                            else break;
                        }
                        
                        if(!created) break; // Error in header, could not create the object
                    }
                    else // The object exists already
                    {
                        QDomElement cont;
                        // If the modified object is a container, modify its contents instead
                        if(object.tagName()=="XContainerResourceDetails")
                        {
                            cont = object;
                            QString href = xom2_getProperty(object, "Value");
                            if(href.startsWith("@href "))
                            {
                                href = href.mid(6);
                                object = xom2_getObject(xomObjects, href);
                            }
                        }
                        else // Else, find its container to modify the flags
                        {
                            objname.chop(2);
                            cont = xom2_getObject(xomObjects, objname);
                        }
                        while(1)
                        {
                            s=ts.readLine();
                            if(s.startsWith("#flags")) // #flag keyword found : Overwrite the default container flags
                            {
                                bool ok;
                                short flags = s.mid(7).toShort(&ok);
                                if(!cont.isNull() && ok && flags>=0 && flags<=255) xom2_setProperty(cont, "Flags", QString::number(flags));
                            }
                            else if(s.startsWith("#")) {}
                            else break;
                        }
                    }
                    
                    // Read and perform the modifications in the tweak file
                    while(!s.isNull())
                    {
                        bool forceAdd = false;
                        QString property = s.section('=',0,0);
                        QString value    = s.section('=',1);
                        if(property.startsWith("+")) {property = property.mid(1);forceAdd = true;}
                        if(forceAdd) xom2_forceAddProperty(object, property, value);
                        else         xom2_setProperty     (object, property, value);
                        s=ts.readLine();
                    }
                    progress.setValue(++filecount);
                    qApp->processEvents();
                }
                // Done ! Writing the new tweak file.
                tweakfile.open(QIODevice::WriteOnly|QIODevice::Text);
                QString xmlstr = doc.toString(2);
                xmlstr.replace("\"", "'");
                tweakfile.write(xmlstr.toAscii());
                tweakfile.close();
                dir.cdUp();
            }
        }
    }
    
    // Success ! Close the window and run the game.
    // Close the dialog, but don't quit the application yet
    qApp->setQuitOnLastWindowClosed(false);
    close();
    
    process = new QProcess();
	connect(process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(gameEnded()));
	
	// Run the game
	process->setWorkingDirectory(gamepath);
    process->start("\""+gamepath+"\\WORMS 4 MAYHEM\"");
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Called when a mod is selected, so the description field is updated

void W4LauncherDialog::updateModInfo(QListWidgetItem* current, QListWidgetItem* previous)
{
    W4ModItem* mod = (W4ModItem*)current;
    QString txt = getlang(LANG_AUTHOR) + mod->author + "\n" + getlang(LANG_DESC) + mod->desc;
    ui.modsDesc->setText(txt);
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Called when a resolution is selected, load its supported refresh rates

void W4LauncherDialog::resolutionChanged() 
{
    ui.refreshCombo->clear();
    ui.refreshCombo->addItems(refreshRates[ui.resolutionCombo->currentIndex()]);
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Open the update checking in default web browser

void W4LauncherDialog::checkUpdates()
{
    QDesktopServices::openUrl(QUrl("http://www.wormsmayhem.com/support.html?subsection=patch&CRC=0x125FFDA8"));
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Move the selected mod up (higher priority)

void W4LauncherDialog::moveUp()
{
    int row = ui.modsList->currentRow();
    if(row>0)
    {
        W4ModItem* item = (W4ModItem*)ui.modsList->takeItem(row);
        if(item)
        {
            ui.modsList->insertItem(row-1, item);
            ui.modsList->setCurrentItem(item);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Move the selected mod down (lower priority)

void W4LauncherDialog::moveDown()
{
    int row = ui.modsList->currentRow();
    if(row<ui.modsList->count()-1)
    {
        W4ModItem* item = (W4ModItem*)ui.modsList->takeItem(row);
        if(item)
        {
            ui.modsList->insertItem(row+1, item);
            ui.modsList->setCurrentItem(item);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////////////
// Slot : Called when the game ends. Restore the old files and exit the launcher

void W4LauncherDialog::gameEnded()
{
    QStringList filter;
    filter << "*.xml.backup";
    
    QDir dir(gamepath + "\\data\\Tweak");
    QStringList twkfiles = dir.entryList(filter, QDir::Files);
    QListIterator<QString> it(twkfiles);
    while (it.hasNext())
    {
        QString tweakfile = it.next();
        QString tweakfile2 = tweakfile;
        tweakfile2.chop(7);
        
        QFile::remove(gamepath + "\\data\\Tweak\\" + tweakfile2);
        QFile::rename(gamepath + "\\data\\Tweak\\" + tweakfile, gamepath + "\\data\\Tweak\\" + tweakfile2);
    }
    
    qApp->exit(0);
}
