#include "xom2.h"
#include <QStringList>
#include <QFile>

bool xom2_load(QDomDocument& doc, QDomElement& root, const QString& path)
{
    QFile file(path);
    if (!file.open(QIODevice::ReadOnly))
       return false;
    if (!doc.setContent(&file))
    {
        file.close();
        return false;
    }
    file.close();
    
    root = doc.documentElement().firstChild().toElement();
    return true;
}

//////////////////////////////////////////////////////////////////////////////////////
// Retourne l'objet de nom "id" dans un tweak (root)
QDomElement xom2_getObject(QDomElement& root, const QString& id)
{
    QDomNode n = root.firstChild();
    while(!n.isNull())
    {
        QDomElement e = n.toElement();
        if(!e.isNull())
        {
            if(e.attribute("id")==id) return e;
        }
        n = n.nextSibling();
    }
    return QDomElement();
}

//////////////////////////////////////////////////////////////////////////////////////
// Cre un objet vide de nom "id", et de classe "classname", et l'ajoute  root
QDomElement xom2_createObject(QDomElement& root, const QString& id, const QString& classname)
{
    QDomElement e = root.ownerDocument().createElement(classname);
    e.setAttribute("id", id);
    root.appendChild(e);
    return e;
}

//////////////////////////////////////////////////////////////////////////////////////
// Cre une copie de l'objet "reference", avec le nom "id", et l'ajoute  root
QDomElement xom2_cloneObject(QDomElement& root, QDomElement& reference, const QString& id)
{
    QDomElement e = root.ownerDocument().importNode(reference,true).toElement();
    e.setAttribute("id", id);
    root.appendChild(e);
    
    return e;
}

//////////////////////////////////////////////////////////////////////////////////////
// Cre une copie de l'objet de nom "refname", avec le nom "id", et l'ajoute  root
QDomElement xom2_cloneObject(QDomElement& root, const QString& refname, const QString& id)
{
    QDomElement e = xom2_getObject(root,refname);
    return xom2_cloneObject(root, e, id);
}


QString xom2_getclass(QDomElement& el)
{
    if(el.hasAttribute("id")) return el.tagName();
    return QString();
}

bool xom2_setProperty(QDomElement& el, const QString& name, const QString& value)
{
    QDomElement e = el.namedItem(name).toElement();
    
    if(e.isNull())
    {
        e = el.ownerDocument().createElement(name);
        el.appendChild(e);
    }
    
    QDomText t;
    QStringList l;
    switch(xom2_valueType(value))
    {
        case NORMAL:
            t = e.firstChild().toText();
            if(t.isNull())
            {
                e.appendChild(e.ownerDocument().createTextNode(value));
            }
            else t.setData(value);
            break;
            
        case COLOR:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("r",l[1]);
            e.setAttribute("g",l[2]);
            e.setAttribute("b",l[3]);
            if(l.count()==5) e.setAttribute("a",l[4]);
            break;
            
        case VECTOR:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("x",l[1]);
            e.setAttribute("y",l[2]);
            e.setAttribute("z",l[3]);
            break;
            
        case HREF:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("href",l[1]);
            break;
            
        default:
            return false;
    }
    return true;
}

bool xom2_setProperty(QDomElement& root, const QString& obj, const QString& name, const QString& value)
{
    QDomElement e = xom2_getObject(root,obj);
    return xom2_setProperty(e,name,value);
}

bool xom2_forceAddProperty(QDomElement& el, const QString& name, const QString& value)
{
    QDomElement e = el.ownerDocument().createElement(name);
    el.appendChild(e);
    
    QDomText t;
    QStringList l;
    switch(xom2_valueType(value))
    {
        case NORMAL:
            t = e.firstChild().toText();
            if(t.isNull())
            {
                e.appendChild(e.ownerDocument().createTextNode(value));
            }
            else t.setData(value);
            break;
            
        case COLOR:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("r",l[1]);
            e.setAttribute("g",l[2]);
            e.setAttribute("b",l[3]);
            if(l.count()==5) e.setAttribute("a",l[4]);
            break;
            
        case VECTOR:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("x",l[1]);
            e.setAttribute("y",l[2]);
            if(l.count()==4) e.setAttribute("z",l[3]);
            break;
            
        case HREF:
            l = value.split(' ',QString::SkipEmptyParts);
            e.setAttribute("href",l[1]);
            break;
            
        default:
            return false;
    }
    return true;
}

bool xom2_forceAddProperty(QDomElement& root, const QString& obj, const QString& name, const QString& value)
{
    QDomElement e = xom2_getObject(root,obj);
    return xom2_forceAddProperty(e,name,value);
}

QString xom2_getProperty(QDomElement& el, const QString& name)
{
    QDomElement e = el.namedItem(name).toElement();
    
    if(!e.isNull())
    {
        if(e.hasAttribute("r")&&e.hasAttribute("g")&&e.hasAttribute("b")&&e.hasAttribute("a"))
        {
            return "@color "+e.attribute("r")+" "+e.attribute("g")+" "+e.attribute("b")+" "+e.attribute("a");
        }
        if(e.hasAttribute("r")&&e.hasAttribute("g")&&e.hasAttribute("b"))
        {
            return "@color "+e.attribute("r")+" "+e.attribute("g")+" "+e.attribute("b");
        }
        if(e.hasAttribute("x")&&e.hasAttribute("y")&&e.hasAttribute("z"))
        {
            return "@vector "+e.attribute("x")+" "+e.attribute("y")+" "+e.attribute("z");
        }
        if(e.hasAttribute("x")&&e.hasAttribute("y"))
        {
            return "@vector "+e.attribute("x")+" "+e.attribute("y");
        }
        if(e.hasAttribute("href"))
        {
            return "@href "+e.attribute("href");
        }
        return e.text();
    }
    return QString();
}

QString xom2_getProperty(QDomElement& root, const QString& obj, const QString& name)
{
    QDomElement e = xom2_getObject(root,obj);
    return xom2_getProperty(e,name);
}

unsigned char xom2_valueType(const QString& name)
{
    if      (name.startsWith("@color" ))
    {
        if(name.split(' ',QString::SkipEmptyParts).count() == 5) return COLOR;
        return ERROR;
    }    
    else if (name.startsWith("@vector"))
    {
        if(name.split(' ',QString::SkipEmptyParts).count() == 4) return VECTOR;
        return ERROR;
    }    
    else if (name.startsWith("@href"  ))
    {
        if(name.split(' ',QString::SkipEmptyParts).count() == 2) return HREF;
        return ERROR;
    }    
    else if (name.startsWith("@"      )) return ERROR;
    return NORMAL;
}
